// -*- C++ -*-
/*!
 * @file  RTC_Adafruit_UltimateGPS.cpp
 * @brief ＵＳＢシリアル接続したAdafruit_UltimateGPSの読み込み
 * @date $Date$
 *
 * $Id$
 */

#include "RTC_Adafruit_UltimateGPS.h"

#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <curses.h>
#include <math.h>
#include <fstream>
int ULTGPS_;
using namespace std;
// Module specification
// <rtc-template block="module_spec">
static const char* rtc_adafruit_ultimategps_spec[] =
  {
    "implementation_id", "RTC_Adafruit_UltimateGPS",
    "type_name",         "RTC_Adafruit_UltimateGPS",
    "description",       "ＵＳＢシリアル接続したAdafruit_UltimateGPSの読み込み",
    "version",           "1.0.0",
    "vendor",            "VenderName",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.PORT", "/dev/ttyUSB0",
    "conf.default.BAUDRATE", "9600",
    // Widget
    "conf.__widget__.PORT", "text",
    "conf.__widget__.BAUDRATE", "text",
    // Constraints
    ""
  };
// </rtc-template>

// char lat/longitude
// ddmm.mmmm →　ddd.dddd　変換
//
double dddmm_to_ddd(char *DO){
    double DO_D,DOmmD,DOddD;
    int DO_I[3];
    DO_D=atof(DO);
    
    DO_I[0] = DO_D;     //dd
    DO_I[1] =DO_D/100;  //mm.mmmm
    
    DOmmD = DO_D-DO_I[1]*100;
    
    //ddd.dddd=dd+mm.mmmm/60
    DOddD=DO_I[1]+DOmmD/60;  
    return DOddD;
}




/*!
 * @brief constructor
 * @param manager Maneger Object
 */
 
 
RTC_Adafruit_UltimateGPS::RTC_Adafruit_UltimateGPS(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_GPS_DataOut("GPS_Data", m_GPS_Data)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
RTC_Adafruit_UltimateGPS::~RTC_Adafruit_UltimateGPS()
{
}



RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("GPS_Data", m_GPS_DataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("PORT", m_PORT, "/dev/ttyUSB0");
  bindParameter("BAUDRATE", m_BAUDRATE, "9600");
  // </rtc-template>
  
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onFinalize()
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onActivated(RTC::UniqueId ec_id)
{
	int i,p_len,ret;
	char buf[55];
	char def[55];
	char* portname;
 	struct termios ULT_ter;
//
//m_SH1_PORTをstringからcharへ
//                     portnameに格納
//m_SH1_PORT コンフィギュレーションパラメータより
//                     /dev/ttyUSBX X=0,1,2...の形
//
	p_len=m_PORT.length();
	portname=new char[p_len+1];
	memcpy(portname, m_PORT.c_str(), p_len+1);
  	ULTGPS_=open(portname, O_RDWR  );
  	
	ret = tcgetattr(ULTGPS_, &ULT_ter);
	ret = tcsetattr(ULTGPS_, TCSANOW, &ULT_ter);
	if (ret){
		printf("ERR tcsetattr\n\r");
	}
	
	 if(m_BAUDRATE.compare("9600")==0){
	        	ULT_ter.c_cflag =(B9600 | CRTSCTS | CS8 | CLOCAL | CREAD);
		//        std::cout<<"Baudrate B9600" <<std::endl;	
	        }else if(m_BAUDRATE.compare("19200")==0){
	        	ULT_ter.c_cflag =(B19200 | CRTSCTS | CS8 | CLOCAL | CREAD);
		//        std::cout<<"Baudrate B19200" <<std::endl;	
	        }else if(m_BAUDRATE.compare("38400")==0){
	        	ULT_ter.c_cflag =(B38400 | CRTSCTS | CS8 | CLOCAL | CREAD);
		 //       std::cout<<"Baudrate B38400" <<std::endl;	
	        }else if(m_BAUDRATE.compare("57600")==0){
	        	ULT_ter.c_cflag =(B57600 | CRTSCTS | CS8 | CLOCAL | CREAD);
		 //       std::cout<<"Baudrate B57600" <<std::endl;	
	        }else{
		        std::cout<<"Baudrate ERR" <<std::endl;	
	        }
        ULT_ter.c_iflag = (IGNPAR | ICRNL);//IGNBRK | IGNPAR;if (ret < 0)
	//printf("ERR");
     //   cfsetispeed(&tio, m_BAUDRATE1);
	ret = tcflush(ULTGPS_, TCIFLUSH);
	if (ret < 0){
			printf("ERR tcflush\n\r");
			
		}

	ret = tcsetattr(ULTGPS_, TCSANOW, &ULT_ter);
	
	
	
	sprintf(buf,"$PMTK314,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0*28\r\n");
	ret=write(ULTGPS_,def,sizeof(buf));
	
	for(i=0;i<strlen(buf);i++){
		def[0]=buf[i];
		ret=write(ULTGPS_,def,sizeof(def));
		if (ret == -1) {
			printf("ERROR\n\r");
		}
	}

	sprintf(buf,"$PMTK314,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0*29\r\n");
	for(i=0;i<strlen(buf);i++){
		def[0]=buf[i];
		ret=write(ULTGPS_,def,sizeof(def));
		if (ret == -1) {
			printf("ERROR\n\r");
		}
	}
	sprintf(buf,"$PMTK220,1000*1F\r\n");
	for(i=0;i<strlen(buf);i++){
		def[0]=buf[i];
		ret=write(ULTGPS_,def,sizeof(def));
		if (ret == -1) {
			printf("ERROR\n\r");
		}
	}
/* */
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onDeactivated(RTC::UniqueId ec_id)
{
  close(ULTGPS_);
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onExecute(RTC::UniqueId ec_id)
{
	int ret;
	char buf[512];
	char GPGGA[15][64];
	memset(&buf, 0, sizeof(buf));
	usleep(11000); 
        ret=read(ULTGPS_, buf,sizeof(buf));
        if (ret == -1) {
			printf("READ_ERROR\n\r");
	//		return RTC::RTC_ERROR;
	}
        cout << ret <<endl;
	printf("--\n\r%s\n\r",buf);
	//0:type 1:time 2:緯度 3:北緯か南緯か 4:経度 5:東経か西経か
	//6:位置特定品質 7：衛星数 8:水平精度低下率 9:アンテナ海抜高さ 10:Ｍ
	//11:ジオイド高 12:Ｍ 13:チェックサム
	sscanf(buf,"%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],,,%[^,]",
		GPGGA[0],GPGGA[1],GPGGA[2],GPGGA[3],GPGGA[4],GPGGA[5],GPGGA[6],GPGGA[7],GPGGA[8],
		GPGGA[9],GPGGA[10],GPGGA[11],GPGGA[12],GPGGA[13]);
        if(strlen(GPGGA[13]) == 0)cout<<"well\n\r"<<endl;
//	cout<<"latitude:"<<GPGGA[2]<<" longitude:"<<GPGGA[4]<<" Satellites:"<<GPGGA[7]<<" altitude:"<<GPGGA[9]<<endl;
	m_GPS_Data.latitude=dddmm_to_ddd(GPGGA[2]);
	m_GPS_Data.longitude=dddmm_to_ddd(GPGGA[4]);
	m_GPS_Data.numSatellites=atoi(GPGGA[7]);
	m_GPS_Data.altitude=atof(GPGGA[9]);
	//m_GPS_Data.timeFromGPS =atof(GPGGA[0]);
	
	//Time
	int GPS_Time_I_M;
	int GPS_Time_I_S;
	double GPS_Time_D;
	double GPS_Time_D_M;
	GPS_Time_D = atof(GPGGA[1]);
	GPS_Time_I_S = (int)GPS_Time_D ;
	GPS_Time_D_M = GPS_Time_D-(int)GPS_Time_D ;
	GPS_Time_I_M = GPS_Time_D_M *1000 ;
        m_GPS_Data.timeFromGPS.sec =GPS_Time_I_S;//高さ  
        m_GPS_Data.timeFromGPS.msec=GPS_Time_I_M;//高さ 
        m_GPS_DataOut.write();
	cout<<"time:"<<GPGGA[1]<<"latitude:"<<m_GPS_Data.latitude<<" longitude:"<<m_GPS_Data.longitude
	<<" Satellites:"<<m_GPS_Data.numSatellites<<" altitude:"<<m_GPS_Data.altitude<<endl;
	 
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Adafruit_UltimateGPS::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}



extern "C"
{
 
  void RTC_Adafruit_UltimateGPSInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtc_adafruit_ultimategps_spec);
    manager->registerFactory(profile,
                             RTC::Create<RTC_Adafruit_UltimateGPS>,
                             RTC::Delete<RTC_Adafruit_UltimateGPS>);
  }
  
};


